@@ -8,7 +8,7 @@ require DynaLoader;
 use vars qw($VERSION @ISA $__import @preload);
 @ISA = qw(DynaLoader);
 sub dl_load_flags { 0x00 }
-$VERSION = '1.49_1';
+$VERSION = '1.49_2';
 bootstrap Prima $VERSION;
 unless ( UNIVERSAL::can('Prima', 'init')) {
 	$::application = 0;
@@ -54,7 +54,7 @@ sub parse_argv
 	@ARGV = parse_argv( @ARGV) if @ARGV and not $skip_argv;
 }
 
-Prima::init($VERSION);
+Prima::init($VERSION) unless $^C;
 
 sub END
 {
@@ -75,7 +75,7 @@ sub import
 		my $module = shift @module;
 		next if $module eq 'Prima' || $module eq '';
 		$module = "Prima::$module" unless $module =~ /^Prima::/;
-		$__import = caller;
+		local $__import = caller;
 		if ( $module) {
 			eval "use $module;";
 			die $@ if $@;
