@@ -236,7 +236,7 @@ sub set_image
 		my $do_cubic;
 
 		if ( $self-> {bitmap}) {
-			$do_cubic = not($img-> monochrome) && $::application-> get_bpp == 8;
+			$do_cubic = $img-> type != dbt::Bitmap && $::application-> get_bpp == 8;
 		} else {
 			$do_cubic = ( $img-> type & im::BPP) > 8;
 		}
@@ -450,12 +450,12 @@ sub stretch      {($#_)?$_[0]-> set_stretch($_[1]):return $_[0]-> {stretch}}
 
 sub PreviewImage_HeaderReady
 { 
-	my ( $self, $image) = @_;
+	my ( $self, $image, $extras) = @_;
 	my $db;
 	eval {
 		$db = Prima::DeviceBitmap-> new(
-			width    => $image-> width,
-			height   => $image-> height,
+			width    => $image->width,
+			height   => $image->height,
 		);
 	};
 	return unless $db;
@@ -463,16 +463,16 @@ sub PreviewImage_HeaderReady
 	$self-> image($db);
         $self-> image-> backColor(0);
         $self-> image-> clear;
-	$self-> {__preview_image} = 1;
+	$self-> {__watch}->{preview} = 1;
 }
 
 sub PreviewImage_DataReady
 { 
 	my ( $self, $image, $x, $y, $w, $h) = @_;
-	return unless $self-> {__preview_image};
+	return unless exists $self-> {__watch} && $self->{__watch}->{preview};
 
 	# do not update if DataReady covers the whole image at once
-	return if $y == 0 and $h == $image-> height;
+	return if $y == 0 and $x == 0 && $h == $image-> height && $w == $image-> width;
 
 	$self-> image-> put_image_indirect( $image, $x, $y, $x, $y, $w, $h, $w, $h, rop::CopyPut);
 	my @r = $self-> point2screen( $x, $y, $x + $w, $y + $h);
@@ -493,32 +493,31 @@ sub watch_load_progress
 		$image-> add_notification( 'HeaderReady', \&PreviewImage_HeaderReady, $self),
 		$image-> add_notification( 'DataReady',   \&PreviewImage_DataReady,   $self)
 		;
-	$self-> {__watch_notifications} = [ @ids ];
+	$self-> {__watch} = {
+		notifications => \@ids,
+	};
 }
 
 sub unwatch_load_progress
 {
 	my ( $self, $clear_image) = @_;
 
-	return unless $self-> {__watch_notifications};
+	return unless $self-> {__watch};
 
 	if ( $self-> {image}) {
 		$self-> {image}-> remove_notification($_) 
-			for @{ $self-> {__watch_notifications} };
+			for @{ $self-> {__watch}-> {notifications} };
 	}
-	delete $self-> {__watch_notifications};
 
-	$clear_image = 1 unless defined $clear_image;
-	if ( $self-> {__preview_image}) {
-		$self-> image( undef) if $clear_image;
-		delete $self-> {__preview_image};
+	if ( $self-> {__watch}-> {preview} && ($clear_image // 1)) {
+		$self-> image( undef);
 	}
+	
+	delete $self-> {__watch};
 }
 
 1;
 
-__DATA__
-
 =pod
 
 =head1 NAME
