@@ -43,10 +43,10 @@ char * keyLayouts[]   = {  "0409", "0403", "0405", "0406", "0407",
 	"0417","0418","041A","041D"
 };
 WCHAR lastDeadKey = 0;
+int          timeDefsCount = 0;
+PItemRegRec  timeDefs = NULL;
 Bool debug = false;
 
-
-
 BOOL APIENTRY
 DllMain( HINSTANCE hInstance, DWORD reason, LPVOID reserved)
 {
@@ -76,8 +76,22 @@ typedef enum _MONITOR_DPI_TYPE {
 #define MONITOR_DEFAULTTONEAREST         2
 #endif
 
-static HRESULT (*SetProcessDpiAwareness)(PROCESS_DPI_AWARENESS)  = NULL;
-static HRESULT (*GetDpiForMonitor)(HMONITOR,MONITOR_DPI_TYPE,UINT*,UINT*) = NULL;
+typedef struct _DWM_BLURBEHIND {
+	DWORD dwFlags;
+	BOOL  fEnable;
+	HRGN  hRgnBlur;
+	BOOL  fTransitionOnMaximized;
+} DWM_BLURBEHIND;
+
+#ifndef DWM_BB_ENABLE
+#define DWM_BB_ENABLE 0x00000001
+#define DWM_BB_BLURREGION 0x00000002
+#define DWM_BB_TRANSITIONONMAXIMIZED 0x00000004
+#endif
+
+static HRESULT (__stdcall *SetProcessDpiAwareness)(PROCESS_DPI_AWARENESS)  = NULL;
+static HRESULT (__stdcall *GetDpiForMonitor)(HMONITOR,MONITOR_DPI_TYPE,UINT*,UINT*) = NULL;
+static HRESULT (__stdcall *DwmEnableBlurBehindWindow)(HWND hWnd, const DWM_BLURBEHIND *pBlurBehind) = NULL;
 
 void
 dpi_change(void)
@@ -91,6 +105,39 @@ dpi_change(void)
 	}
 }
 
+Bool
+set_dwm_blur( HWND win, int enable, HRGN mask, int transition_on_maximized)
+{
+	DWM_BLURBEHIND b;
+
+	if ( !win )
+		return DwmEnableBlurBehindWindow != NULL;
+
+	if ( !DwmEnableBlurBehindWindow ) return;
+
+	b. dwFlags = 0;
+	if ( enable >= 0 ) {
+		b. dwFlags |= DWM_BB_ENABLE;
+		b. fEnable = enable;
+	}
+	if (( HRGN) mask != (HRGN)-1) {
+		b. dwFlags |= DWM_BB_BLURREGION;
+		b. hRgnBlur = mask;
+	}
+	if ( transition_on_maximized >= 0 ) {
+		b. dwFlags |= DWM_BB_TRANSITIONONMAXIMIZED;
+		b. fTransitionOnMaximized = transition_on_maximized;
+	}
+
+	if ( DwmEnableBlurBehindWindow(win, &b) != S_OK ) {
+		apiErr;
+		apcErrClear;
+		return false;
+	}
+
+	return true;
+}
+
 static void
 load_function(HMODULE module, void ** ptr, const char * name)
 {
@@ -188,22 +235,29 @@ window_subsystem_init( char * error_buf)
 	guts. displayResolution. x = GetDeviceCaps( dc, LOGPIXELSX);
 	guts. displayResolution. y = GetDeviceCaps( dc, LOGPIXELSY);
 
+	/* Win7 DWM */
+#define LOAD_FUNC(m,f) load_function(m, (void**) &f, #f)
+	if ( os.dwMajorVersion >= 5) {
+		HMODULE dwm = LoadLibrary("DWMAPI.DLL");
+		if ( dwm ) {
+			LOAD_FUNC(dwm, DwmEnableBlurBehindWindow);
+		}
+	}
+
 	/* Win8 - high dpi awareness stuff */
 	if (( os.dwMajorVersion > 5) || (os.dwMajorVersion == 5 && os.dwMinorVersion > 1)) {
 		HMODULE shcore = LoadLibrary("SHCORE.DLL");
 		if ( shcore ) {
-#define LOAD_FUNC(f) load_function(shcore, (void**) &f, #f)
-			LOAD_FUNC(SetProcessDpiAwareness);
-			LOAD_FUNC(GetDpiForMonitor);
-#undef LOAD_FUNC
+			LOAD_FUNC(shcore, SetProcessDpiAwareness);
+			LOAD_FUNC(shcore, GetDpiForMonitor);
 		}
 		if (
 			SetProcessDpiAwareness && GetDpiForMonitor &&
 			(SetProcessDpiAwareness( PROCESS_PER_MONITOR_DPI_AWARE) == S_OK )
 		)
 			dpi_change();
-	
 	}
+#undef LOAD_FUNC
 
 	{
 		LOGFONT lf;
@@ -332,6 +386,8 @@ static Bool myfont_cleaner( void * value, int keyLen, void * key, void * dummy)
 void
 window_subsystem_done()
 {
+	free( timeDefs);
+	timeDefs = NULL;
 	list_destroy( &guts. files);
 
 	if ( guts. socketMutex) {
@@ -974,16 +1030,6 @@ AGAIN:
 		if ( sys sizeLockLevel == 0 && var stage <= csNormal)
 			var virtualSize = sys lastSize;
 		break;
-	case WM_TIMER:
-		{
-			int id = mp1 - 1;
-			if ( id >= 0 && id < sys timeDefsCount) ev. gen. H = ( Handle) sys timeDefs[ id]. item;
-			if ( ev. gen. H) {
-				v = ( PWidget)( self = ev. gen. H);
-				ev. cmd = cmTimer;
-			}
-		}
-		break;
 	case WM_WINDOWPOSCHANGING:
 		{
 			LPWINDOWPOS l = ( LPWINDOWPOS) mp2;
@@ -1239,41 +1285,47 @@ LRESULT CALLBACK generic_frame_handler( HWND win, UINT  msg, WPARAM mp1, LPARAM
 		break;
 // case WM_SYSCHAR:return 1;
 	case WM_TIMER:
-		if ( mp1 == TID_USERMAX)
-		// application local timer
-		{
-		POINT p;
-		HWND wp;
-		if ( lastMouseOver && !GetCapture() && ( PObject( lastMouseOver)-> stage == csNormal))
-		{
-			HWND desktop = HWND_DESKTOP;
-			GetCursorPos( &p);
-			wp = WindowFromPoint( p);
-			if ( wp) {
-				POINT xp = p;
-				MapWindowPoints( desktop, wp, &xp, 1);
-				wp = ChildWindowFromPointEx( wp, xp, CWP_SKIPINVISIBLE);
-			} else
-				wp = ChildWindowFromPointEx( wp, p, CWP_SKIPINVISIBLE);
-			if ( wp != ( HWND)(( PWidget) lastMouseOver)-> handle)
-			{
-				HWND old = ( HWND)(( PWidget) lastMouseOver)-> handle;
-				Handle s;
-				lastMouseOver = nilHandle;
-				SendMessage( old, WM_MOUSEEXIT, 0, 0);
-				s = hwnd_to_view( wp);
-				if ( s && ( HWND)(( PWidget) s)-> handle == wp)
+		if ( mp1 == TID_USERMAX) {
+			POINT p;
+			HWND wp;
+			if ( lastMouseOver && !GetCapture() && ( PObject( lastMouseOver)-> stage == csNormal)) {
+				HWND desktop = HWND_DESKTOP;
+				GetCursorPos( &p);
+				wp = WindowFromPoint( p);
+				if ( wp) {
+					POINT xp = p;
+					MapWindowPoints( desktop, wp, &xp, 1);
+					wp = ChildWindowFromPointEx( wp, xp, CWP_SKIPINVISIBLE);
+				} else
+					wp = ChildWindowFromPointEx( wp, p, CWP_SKIPINVISIBLE);
+				if ( wp != ( HWND)(( PWidget) lastMouseOver)-> handle)
 				{
-					MapWindowPoints( desktop, wp, &p, 1);
-					SendMessage( wp, WM_MOUSEENTER, 0, MAKELPARAM( p. x, p. y));
-					lastMouseOver = s;
-				} else if ( guts. mouseTimer) {
-					guts. mouseTimer = 0;
-					if ( !KillTimer( dsys(application)handle, TID_USERMAX)) apiErr;
+					HWND old = ( HWND)(( PWidget) lastMouseOver)-> handle;
+					Handle s;
+					lastMouseOver = nilHandle;
+					SendMessage( old, WM_MOUSEEXIT, 0, 0);
+					s = hwnd_to_view( wp);
+					if ( s && ( HWND)(( PWidget) s)-> handle == wp)
+					{
+						MapWindowPoints( desktop, wp, &p, 1);
+						SendMessage( wp, WM_MOUSEENTER, 0, MAKELPARAM( p. x, p. y));
+						lastMouseOver = s;
+					} else if ( guts. mouseTimer) {
+						guts. mouseTimer = 0;
+						if ( !KillTimer( dsys(application)handle, TID_USERMAX)) apiErr;
+					}
+				}
+			}
+			return 0;
+		} else {
+			int id = mp1 - 1;
+			if ( id >= 0 && id < timeDefsCount) {
+				ev. gen. H = ( Handle) timeDefs[ id]. item;
+				if ( ev. gen. H) {
+					v = ( PWidget)( self = ev. gen. H);
+					ev. cmd = cmTimer;
 				}
 			}
-		}
-		return 0;
 		}
 		break;
 	case WM_GETMINMAXINFO:
